//===================================================
//
// File Model.java
// jlouie 12.96
// Encapsulates application logic and data
// Supplied "as is"
// No warranty is expressed or implied
// This code is for instructional use only
//
//===================================================

import java.util.*;
import java.awt.*;

public class Model {

    // array of double, stores numeric input
    private double         arrayDb[]= new double[4];
    private double         dbResult= 0;
    private TextField[]    arrayTextFields= new TextField[4];
    private String         message="";
    private Toolkit        tool= null;

   //==================================
   // * ParseString *
   //
   // Strips the input string of commas
   //
   // Use a StringBuffer to avoid multiple
   // memory allocations
   //
   //==================================

   private String ParseString(String inString){
      StringBuffer   outString= new StringBuffer(inString.length());

      for (int i=0; i<inString.length(); i++)
      {
         if (inString.charAt(i) != ',')
            {outString.append(inString.charAt(i));}
      }
      return outString.toString();
   }

   //==================================
   // * DoAlgo * CalcInt *
   //
   // The actual amortization algorithm:
   //
   // m= P*i/(1-(1+i)^-N)
   // i=r/1200
   //
   // solving for i is non_trivial
   // Thanks to Dr. W. Carlini (& Newton) for this formula!
   //
   //==================================

   // Newton's theorem to the rescue.

   // returns 0 on error
   private double CalcInt(double P, double N, double m){

      double temp=(m/P), answer=(m/P), diff=100, num=0, den=0, accuracy=.00001;
      int i, maxIterations= 1000;
      
      try{
         for (i=0; ((diff>accuracy) && (i<maxIterations)); i++){
            temp= answer;
            num= (P*temp/m)+Math.pow((1+temp), -N)-1;
            den= (P/m)-N*Math.pow((1+temp),(-N-1));
            //if (den==0) throw new ArithmeticException(); // not needed
            answer= temp- (num/den);
            diff= answer- temp;
            if (diff<0) diff= -diff;
         }
         answer *= 1200;
         if ((answer < 0) || (Double.isNaN(answer)) || (i == maxIterations)){
            throw new ArithmeticException();
         }
      }
      catch (ArithmeticException e) {answer= 0;}
      return answer;
   }

   private String DoAlgo(int field) {
      String   outString;
      double   db=0, temp= 0;
      double   P= arrayDb[0];          // principal
      double   i= (arrayDb[1]/1200);   // monthly percentage rate
      double   N= arrayDb[2];          // loan period months
      double   m= arrayDb[3];          // monthly payments

      if (field<4){
         try{
            switch (field){
               case 0: // Principal
                  db= 1+i;
                  db= 1/Math.pow(db, N);
                  db= ((1-db)/i)*m;
                  break;
               case 1:  // Annual Interest
                  if ((N*m)<P) throw new ArithmeticException();
                  db= CalcInt(P, N, m);
                  break;
               case 2:  // Loan Period
                  db= (1-(P*i/m));
                  db= Math.log(db);
                  temp= 1+i;
                  db= -db/Math.log(temp);
                  break;
               case 3:  // Monthly Payments
                  db= 1+i;
                  db= 1/Math.pow(db, N);
                  db= (P*i)/(1-db);
                  break;
               default:;
               }
         }
         catch (ArithmeticException e) {db=0;}
      }
      if (Double.isNaN(db)) db= 0;
      if (db == 0) return "Input Error";
      else return Double.toString(db);
   }

   //==================================
   // * DoCalc *
   //
   // Check for valid input, update the
   // window fields and then DoAlgo()
   //
   //==================================

   public String DoCalc(TextField[] arrayTextFields)
   {
      String   strArray[], strResult= "", tempString="";
      Double   db;
      int      count= 0, field= 0;
      boolean  isError= false;
      TextField target;

      if ((arrayTextFields == null)||(arrayTextFields.length<4)) {
         return "Error in DoCalc.";
      }
      this.arrayTextFields= arrayTextFields;
      strArray= new String[4];

      for (int i=0; i<4; i++){
         strArray[i]= arrayTextFields[i].getText();
         try{
            tempString= ParseString(strArray[i]);  // strip commas
            db= Double.valueOf(tempString);        // get Double, may throw exception
            arrayDb[i]= db.doubleValue();          // convert Double to double
            if (arrayDb[i]<0) throw new NumberFormatException();  // negative #

            switch (i){
               case 0:  // Principle
               break;
               case 1:  // Annual Interest
                  if (arrayDb[i]>100)
                     {throw new NumberFormatException();} // over 100%
               break;
               case 2:  // Months
                  if ((int)arrayDb[i] != arrayDb[i])
                     {throw new NumberFormatException();}  // not an integer
               break;
               case 3:  // Monthly Payment
               break;
            }
            arrayTextFields[i].setText(Double.toString(arrayDb[i]));
         }
         catch (NumberFormatException e){
            arrayDb[i]= 0;
            isError= true;
            arrayTextFields[i].setText("Error");
         }

         if (arrayDb[i]==0) {
            count +=1;  //count zero entries
            field= i;   //last zero text field
         }
      }


      if (isError) {
         //DoDialogError("Invalid Entry");
         message= "Invalid Entry";
      }
      else {
         if (count>1) {
            //DoDialogError("Too Many Zero Entries");
            message= "Too Many Zero Entries";
         }
         else {
            if (count<1) {
               //DoDialogError("One Entry Must Be 0");
               message= "One Entry Must Be 0";
            }
            else {
               strResult= DoAlgo(field);  // finally, do the math!
               arrayTextFields[field].setText(strResult);
               message= "Done";
            }
         }
      }
      if (! message.equals("Done")) {
         Beep();
      }
      return message;
   }
   
   private void Beep() {
      if (tool != null) {
         tool.beep();
      }
      else {
         tool= Toolkit.getDefaultToolkit();
         if (tool != null) {
            tool.beep();
         }
      }                
   }


}
