/*
 * @(#)DateCalc.java	1.00 97/02/23
 *
 * Copyright (c) 1997 Fabula Software GmbH. All Rights Reserved.
 *
 * <a href=http://www.fabula.com>Fabula Software GmbH</a>
 * <a href=mailto:info@fabula.com>eMail</a>
 */

package fabula.util;

import  java.util.*;

/**
 * The class <code>DateCalc</code> defines several methods for calculating 
 * dates and similar stuff. For example you'll find a method for calculate 
 * easter, the week number, begin and end of quarter, etc.
 * <p>
 *
 * @author  <a href="mailto:stefc@fabula.com">Stefan Bther</a>
 * @company <a href="http://www.fabula.com">Fabula Software</a>
 * @version 1.00, 31 Mar 1997
 */

public class DateCalc {

    public static final byte JANUARY   =  1;
    public static final byte FEBRUARY  =  2;
    public static final byte MARCH	   =  3;
    public static final byte APRIL	   =  4;
    public static final byte MAY	   =  5;
    public static final byte JUNE	   =  6;
    public static final byte JULY	   =  7;
    public static final byte AUGUST	   =  8;
    public static final byte SEPTEMBER =  9;
    public static final byte OCTOBER   = 10;
    public static final byte NOVEMBER  = 11;
    public static final byte DECEMBER  = 12;

    public static final byte SUNDAY		= 0;
    public static final byte MONDAY		= 1;
    public static final byte TUESDAY	= 2;
    public static final byte WEDNESDAY	= 3;
    public static final byte THURSDAY	= 4;
    public static final byte FRIDAY		= 5;
    public static final byte SATURDAY	= 6;

	
    static private long ONEDAY = (60 * 60 * 24 * 1000);

	/**
	 * Don't let anyone instantiate this class.
	 */
	private DateCalc() {}


	/**
     * Returns the easter date for the <code>year</code> values.
     *
     * @param   year   the <code>year</code> for that easter should be calculated.
     * @return  the easter sunday for the given year
	 */
	public static Date Easter(int year) {

		int k1 = 24;
		int k2 =  5;

		if (year < 1900)
			year += 1900;

		int r1 = year % 19;
		int r2 = year %  4;
		int r3 = year %  7;
		int r4 = (19 * r1 + k1) % 30;
		int r5 = ( 2 * r2 + 4 * r3 + 6 * r4 + k2) % 7;

		int day;
		int month;

		int a = 22 + r4 + r5;

		if (a < 32 ) {
			month = 2;
			day = a;
		} 
		else 
		{
		  a = r4 + r5 - 9;
		  month = 3;
		  if (a==26)
	        day = 19;
	      else if (a != 25)
	        day = a;
	      else if  ((r4==18) || (r5 > 10))
	        day = 18;
	      else
	        day = 25;
		}

		return new Date(year-1900, month, day );
	}

	/**
     * Returns a date from a Delphi like format 
     *
     * @param   year   the year that should be used
	 * @param   month  the month that should be used
	 * @param   day    the day that should be used
     * @return  the easter sunday for the given year
	 */
	public static Date Encode (int year, int month, int day ) {
        if (year > 1900) year -= 1900;

		return new Date(year, --month, day); 
	}

	/**
     * Returns the year from a given date 
     *
     * @param   D   the date that should be used 
	 * @return  the year for the date 
	 */
	public static int Year (Date D) {
        return D.getYear()+1900; 
	}

	/**
     * Returns the month from a given date 
     *
     * @param   D   the date that should be used 
	 * @return  the month for the date 
	 */
	public static int Month (Date D) {
        return D.getMonth()+1; 
	}

	/**
     * Returns the day within the month from a given date 
     *
     * @param   D   the date that should be used 
	 * @return  the day for the date 
	 */
	public static int Day (Date D) {
        return D.getDate(); 
	}

	/**
     * Returns the weekday from from a given date 
     *
     * @param   D   the date that should be used 
	 * @return  the weekday for the date 
	 */
	public static int DayOfWeek (Date D) {
        return D.getDay(); 
	}

	/**
     * Returns the first day of a year (New Year)
     *
     * @param   D   the date that should be used 
	 * @return  the date of the first day in the year 
	 */
	public static Date BeginOfYear (Date D) {
		return Encode(Year(D),1,1); 
	}

	/**
     * Returns the last day of a year (Silverster)
     *
     * @param   D   the date that should be used 
	 * @return  the date of the last day in the year 
	 */
	public static Date EndOfYear (Date D) {
		return Encode(Year(D),12,31); 
	}

	/**
     * Returns the first day of a month 
     *
     * @param   D   the date that should be used 
	 * @return  the date of the first day in the month
	 */
	public static Date BeginOfMonth (Date D) {
		return Encode(Year(D),Month(D),1); 
	}

	/**
     * Returns the last day of a month 
     *
     * @param   D   the date that should be used 
	 * @return  the date of the last day in the month
	 */
	public static Date EndOfMonth (Date D) {
		int M = Month(D);
		int Y = Year(D); 
		if (M == DECEMBER) {
			Y++;
			M = JANUARY;
		} else {
			M++; 
		}
		return add( Encode(Y,M,1), -1); 
	}

	/**
     * Adds a number of days to a given date 
     *
     * @param   D     the to which should be added 
	 * @param   aDays the number of days that should be added 
	 * @return  the new calculated date 
	 */
	public static Date add(Date D, int aDays) {
		return new Date(Date.UTC(D.getYear(), D.getMonth(), D.getDate(),0,0,0) +
                    (aDays * ONEDAY) );
    }

	/**
     * Converts a date into UTC format 
     *
     * @param   D     the date which should be converted to UTC 
	 * @return  a date in UTC format 
	 */
	private static long ToInt( Date D) {
		return Date.UTC(D.getYear(), D.getMonth(), D.getDate(),0,0,0);
	}

	/**
     * Returns the number of the day within the year 
     *
     * @param   D     the date which should be used
	 * @return  the day within the year 
	 */
	public static int DayOfYear(Date D) {
		return (int)((ToInt(D)-ToInt(BeginOfYear(D))) / ONEDAY)+1;
	}

	/**
     * Calculates the number of the week of a date 
     *
     * @param   D     the date which should be used 
	 * @return  the number of the week
	 */
	public static int WeekOfYear(Date D) {

		byte t1[] = { -1, 0, 1, 2,  3, -3, -2 }; 
		byte t2[] = { -4, 2, 1, 0, -1, -2, -3 };
		
		Date NewYear = BeginOfYear(D); 
		
		int doy1 = DayOfYear(D) + t1[NewYear.getDay()];
		int doy2 = DayOfYear(D) + t2[D.getDay()];
		
		if (doy1 <= 0) {
			return WeekOfYear(add(NewYear,-1));
		} else if (doy2 >= DayOfYear(EndOfYear(NewYear))) {
			return 1; 
		} else {
			return (doy1-1) / 7+1; 
		}
	}

	/**
     * Check if date is in a leap year or not 
     *
     * @param   D     the date which should be used 
	 * @return  logical value indicated if the date is a leapyear
	 */
	public static boolean LeapYear(Date D) {

		int aYear = Year(D); 
		return  ((aYear % 4  ) == 0 ||   //
				 (aYear % 100) == 0 &&   // Centuries aren't
				 (aYear % 400) != 0);    // Except every 4th
	}


	/**
     * returns the begin of a quarter from a date 
     *
     * @param   D     the date which should be used 
	 * @return  date with the begin of the quarter 
	 */
	public static Date BeginOfQuarter(Date D) {
		int aMonth = Month(D)-1;
		return Encode(Year(D),((aMonth / 3)*3)+1,1);
	}

	/**
     * returns the end of a quarter from a date 
     *
     * @param   D     the date which should be used 
	 * @return  date with the end of the quarter 
	 */
	public  static Date EndOfQuarter(Date D) {
		return add(BeginOfQuarter(add(BeginOfQuarter(D),3*31)),-1);
	}

	/**
     * determine the first date in a week 
     *
     * @param   D     the date which should be used 
     * @param   Weekday  the weekday which each week starts (SUNDAY or MONDAY) 
	 * @return  first date of the week 
	 */
	public static Date BeginOfWeek(Date D, int Weekday) {
		Date Result = D; 
		while ( DayOfWeek(Result) != Weekday) {
			Result = add(Result, -1);
		}
		return Result;
	}

	/**
     * returns the number of days in a month 
     *
     * @param   D     the date which should be used 
	 * @return  the number of days in the month 
	 */
	public static int DaysInMonth ( Date D ) {
		byte DaysPerMonth[] = {31,28,31,30,31,30,31,31,30,31,30,31};
		int  aMonth = Month(D); 
		int	 Result = DaysPerMonth[aMonth-1];
		if ((aMonth==FEBRUARY) || (LeapYear(D))) Result++;  
		return Result;
	}

	/**
     * calculates a new date with Quicken key schema 
     *
     * @param   D     the date which should be used 
     * @param   Key   key that is pressed for date calculating 
	 * @return  the new calculated date 
	 */
	public static Date Quicken( Date D, char Key) {
		
		switch (Key) {
		case '+'		:	return add(D,+1);
		case '-'		:	return add(D,-1);
		case 'T'    	:	return new Date();
		case 'Y'      	:	if (BeginOfYear(D) != D) { 
								return BeginOfYear(D);
							} else {
								return BeginOfYear(add(D,-1));
							}
		case 'R'		:	if (EndOfYear(D) != D) { 
								return EndOfYear(D);
							} else {
								return EndOfYear(add(D,+1));
							}
		case 'M'		:	if (BeginOfMonth(D) != D) { 
								return BeginOfMonth(D);
							} else {
								return BeginOfMonth(add(D,-1));
							}
		case 'H'		:	if (EndOfMonth(D) != D) { 
								return EndOfMonth(D);
							} else {
								return EndOfMonth(add(D,+1));
							}
		case 'Q'		:	return BeginOfQuarter(D); 
		case 'U'		:	return EndOfQuarter(D); 

		} // switch 
		
		return D;
	}


}
