/*
 * @(#)IniFile.java	
 *
 * Copyright (c) 1997 Fabula Software GmbH. All Rights Reserved.
 *
 * <a href=http://www.fabula.com>Fabula Software GmbH</a>
 * <a href=mailto:info@fabula.com>eMail</a>
 */

package fabula.util;

import  java.io.*;

import  fabula.util.Str;

/**
 * The class <code>IniFile</code> make it possible to access 
 * *.INI files from Java 
 * <p>
 *
 * @author  <a href="mailto:stefc@fabula.com">Stefan Bther</a>
 * @company <a href="http://www.fabula.com">Fabula Software</a>
 * @version 1.00, 31 Mar 1997
 */

public class IniFile {

	public  static String	EXTENSION = ".INI";
	
	private File			fFile;
	private String			fBuffer; 
	private int				fHash;

		
	public IniFile ( String aFileName ) throws IOException {

		fFile = new File(aFileName); 
		if (fFile.exists()) {
			FileInputStream aStream = new FileInputStream(fFile);

			try {
				byte aBytes[] = new byte[(int)fFile.length()];
				aStream.read(aBytes);
				fBuffer = new String(aBytes, 0); 
		
			} 
			finally 
			{
				aStream.close();
			} 
		} else {
			fBuffer = new String("[Empty]");
		}

		fHash = fBuffer.hashCode();
	}

	public void Close() throws IOException {

		if (fHash != fBuffer.hashCode()) {
			FileOutputStream aStream = new FileOutputStream(fFile);
			try {
				byte aBytes[] = new byte[fBuffer.length()];
				fBuffer.getBytes(0,fBuffer.length(), aBytes,0);
				aStream.write(aBytes); 
			} finally {
				aStream.close();
			}
		}

	}


    public void finalize() throws IOException  {
		Close();
    }

	public String ReadString ( String aSection, String aItem, String aDefault) {

		Range aRange = getLineRange(aSection, aItem);
		if (aRange.Start != -1) {
			String aLine = fBuffer.substring(aRange.Start,aRange.End).trim();
			return getValue(aLine);
		} else {
			return aDefault;
		}
	}

	public void WriteString( String Section, String Item, String Value ) {

		Range aRange = getSectionRange(Section);

		if (aRange.Start == -1) {
			fBuffer = fBuffer.concat(Str.CRLF+'['+Section+']'+Str.CRLF+Item+'='+Value); 
		} else {
			Range aLineRange = getLineRange(Section, Item);
			if (aLineRange.Start != -1) {
				fBuffer = fBuffer.substring(0,aLineRange.Start)+
						Item+'='+Value+
						fBuffer.substring(aLineRange.End,fBuffer.length());
			} else {
				fBuffer = fBuffer.substring(0,aRange.End)+
						Item+'='+Value+Str.CRLF+
						fBuffer.substring(aRange.End,fBuffer.length());
			}
		}
	}


	public int ReadInteger( String aSection, String aItem, int aDefault) {
		return Integer.parseInt(ReadString( aSection, aItem, Integer.toString(aDefault))); 
	}

	public void WriteInteger( String Section, String Item, int Value ) {
		WriteString(Section, Item, Integer.toString(Value));
	}

	public boolean ReadBool( String aSection, String aItem, boolean aDefault) {
		Boolean aBool = new Boolean(aDefault);
		return Boolean.valueOf(ReadString( aSection, aItem, aBool.toString())).booleanValue();
	}

	public void WriteBool( String Section, String Item, boolean Value ) {
		Boolean aBool = new Boolean(Value);
		WriteString( Section, Item, aBool.toString()); 
	}

	public void EraseSection( String Section ) {
		
		Range aRange = getSectionRange(Section);
		if (aRange.Start != -1 ) {
			fBuffer = fBuffer.substring(0,aRange.Start)+
						fBuffer.substring(aRange.End,fBuffer.length());
		}
	}

	private boolean isItem(String aLine, String aItem) {

		int aPos = aLine.indexOf('=');
		if (aPos != -1) {
			return aItem.equalsIgnoreCase(getItem(aLine));
		} else {
			return false; 
		}
	}

	private String getItem(String aLine) {
		int aPos = aLine.indexOf('=');
		if (aPos != -1) {
			return aLine.substring(0,aPos);
		} else {
			return "";
		}
	}

	private String getValue(String aLine) {
		int aPos = aLine.indexOf('=');
		if (aPos != -1) {
			return aLine.substring(aPos+1,aLine.length());
		} else {
			return "";
		}
	}

	private Range getSectionRange(String aSection) {

		String aComp;

		int aStart = -1;
		int aEnd = -1;

		int aPos = fBuffer.indexOf('[');

		while (aPos != -1) {
			aEnd = fBuffer.indexOf(']',aPos); 

			if (aEnd != -1) {
				aPos++;
				aComp = fBuffer.substring(aPos,aEnd);
				if (aComp.equalsIgnoreCase(aSection)) {
					aStart = aPos;
					break;
				}
				aPos = aEnd;
			}
			aPos = fBuffer.indexOf('[',aPos);
		}
		
		if ( aStart != -1 ) {
			aEnd = fBuffer.indexOf('[',aStart); 
			if (aEnd == -1) aEnd=fBuffer.length(); 
		}

		return new Range(aStart,aEnd);
	}

	private Range getLineRange(String aSection, String aItem) {
		String aLine;

		Range aRange = getSectionRange(aSection); 
		int aPos = aRange.Start;
		int aEnd = aRange.End;
	
		if (aPos != -1 ) {
			int aEol; 
			aPos = fBuffer.indexOf(Str.LF,aPos);
			while ((aPos != -1) && (aPos < aEnd)) { 
				aPos++;
				aEol = fBuffer.indexOf(Str.LF,aPos);
				if (aEol==-1) aEol=fBuffer.length();

				if (fBuffer.charAt(aEol-1)==Str.CR) aEol--;

				aLine = fBuffer.substring(aPos,aEol); 
				if (isItem(aLine, aItem)) {
					return new Range(aPos,aEol);
				}
				aPos = fBuffer.indexOf(Str.LF,aEol);
			}
		}

		return new Range (-1,-1);
	}

	




}

// private class represents only a range  
class Range {

	public int Start ;
	public int End;

	public Range (int aStart, int aEnd) {
		Start = aStart;
		End = aEnd;
	} 

}
